@@ -1,13 +1,15 @@
 forest <- function(x,
-                   byvar, bylab,
-                   print.byvar=TRUE,
+                   byvar=x$byvar,
+                   bylab=x$bylab,
+                   print.byvar=x$print.byvar,
                    sortvar,
                    studlab=TRUE,
                    ##
                    level=x$level,
                    level.comb=x$level.comb,
                    ##
-                   comb.fixed=FALSE, comb.random=FALSE, overall=comb.fixed|comb.random,
+                   comb.fixed=x$comb.fixed, comb.random=x$comb.random,
+                   overall=TRUE,
                    text.fixed="Fixed effect model",
                    text.random="Random effects model",
                    lty.fixed=2, lty.random=3,
@@ -15,7 +17,7 @@ forest <- function(x,
                    xlab=NULL, xlab.pos=ref, xlim,
                    ##
                    allstudies=TRUE,
-                   weight=ifelse(comb.random & !comb.fixed, "random", "fixed"),
+                   weight,
                    ##
                    ref=ifelse(x$sm %in% c("RR", "OR", "HR"), 1, 0),
                    ##
@@ -43,13 +45,47 @@ forest <- function(x,
                    col.by="darkgray"){
   
   
-  x.name <- deparse(substitute(x))
-  byvar.name <- deparse(substitute(byvar))
-  
   if (!inherits(x, "meta"))
     stop("Argument 'x' must be an object of class \"meta\"")
   
   
+  x.name <- deparse(substitute(x))
+  
+  byvar.name <- deparse(substitute(byvar))
+  ##
+  if (!missing(byvar) & length(x$byvar)==0){
+    if (!is.null(x[[byvar.name]]))
+      byvar <- x[[byvar.name]]
+  }
+  
+  
+  if (length(comb.fixed)==0){
+    comb.fixed <- FALSE
+  }
+  ##
+  if (length(comb.random)==0){
+    comb.random <- FALSE
+  }
+  ##
+  if (length(print.byvar)==0){
+    print.byvar <- TRUE
+  }
+
+  
+  if (length(level)==0){
+    warning("level set to 0.95")
+    level <- 0.95
+  }
+  ##
+  if (length(level.comb)==0){
+    if (comb.fixed | comb.random)
+      warning("level.comb set to 0.95")
+    level.comb <- 0.95
+  }
+
+  if (missing(weight))
+    weight <- ifelse(comb.random & !comb.fixed, "random", "fixed")
+  
   wcalc <- function(x)
     max(unit(rep(1, length(x)), "grobwidth", x))
   
@@ -544,13 +580,13 @@ forest <- function(x,
     n.stud <- x$k   # number of trials combined in meta-analysis
 
   
-  by <- !missing(byvar)
+  by <- length(byvar)>0
   sort <- !missing(sortvar)
   
-  if (by)
-    if (!is.null(x[[byvar.name]]))
-      byvar <- x[[byvar.name]]
-
+  ##  if (by)
+  ##    if (!is.null(x[[byvar.name]]))
+  ##      byvar <- x[[byvar.name]]
+  
   if (by & (inherits(x, "metainf")| inherits(x, "metacum")))
     stop("Use of 'byvar' not possible for 'metainf' or 'metacum' object.") 
   
@@ -654,17 +690,6 @@ forest <- function(x,
   by.levs <- unique(byvar)
   
   
-  if (length(level)==0){
-    warning("level set to 0.95")
-    level <- 0.95
-  }
-  ##
-  if (length(level.comb)==0){
-    if (comb.fixed | comb.random)
-      warning("level.comb set to 0.95")
-    level.comb <- 0.95
-  }
-  
   sum.meta <- summary(x, level=level, level.comb=level.comb, warn=FALSE)
   ##
   TE    <- sum.meta$study$TE
