@@ -6,7 +6,12 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
                            "OR", "RR"),
                     incr=0.5, allincr=FALSE, addincr=FALSE, allstudies=FALSE,
                     MH.exact=FALSE, RR.cochrane=FALSE,
-                    warn=TRUE){
+                    level=0.95, level.comb=level,
+                    comb.fixed=TRUE, comb.random=TRUE,
+                    title="", complab="", outclab="",
+                    byvar, bylab, print.byvar=TRUE,
+                    warn=TRUE
+                    ){
   
   
   if (is.null(data)) data <- sys.frame(sys.parent())
@@ -16,7 +21,8 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
   mf <- match.call()
   mf$data <- mf$subset <- mf$method <- mf$sm <- NULL
   mf$incr <- mf$allincr <- mf$addincr <- mf$allstudies <- NULL
-  mf$MH.exact <- mf$RR.cochrane <- mf$warn <- NULL
+  mf$MH.exact <- mf$RR.cochrane <- NULL
+  mf$level <- mf$level.comb <- mf$warn <- NULL
   mf[[1]] <- as.name("data.frame")
   mf <- eval(mf, data)
   ##
@@ -28,7 +34,8 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
   mf2$studlab <- NULL 
   mf2$data <- mf2$method <- mf2$sm <- NULL
   mf2$incr <- mf2$allincr <- mf2$addincr <- mf2$allstudies <- NULL
-  mf2$MH.exact <- mf2$RR.cochrane <- mf2$warn <- NULL
+  mf2$MH.exact <- mf2$RR.cochrane <- NULL
+  mf2$level <- mf2$level.comb <- mf2$warn <- NULL
   mf2[[1]] <- as.name("data.frame")
   ##
   mf2 <- eval(mf2, data)
@@ -40,16 +47,16 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     else
       mf <- mf[mf2$subset,]
   ##
-  event.e <- as.numeric(mf$event.e)
-  n.e <- as.numeric(mf$n.e)
-  event.c <- as.numeric(mf$event.c)
-  n.c <- as.numeric(mf$n.c)
+  event.e <- mf$event.e
+  n.e     <- mf$n.e
+  event.c <- mf$event.c
+  n.c     <- mf$n.c
   ##
   if (!missing(studlab))
     studlab <- as.character(mf$studlab)
   else
     studlab <- row.names(mf)
-
+  
   
   k.all <- length(event.e)
   ##
@@ -73,6 +80,31 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
   ##
   if (length(studlab) != k.all)
     stop("Number of studies and labels are different")
+  ##
+  if (!is.numeric(incr)){
+    ##
+    iincr <- charmatch(tolower(incr),
+                       c("tacc"), nomatch = NA)
+    ##
+    if(is.na(iincr))
+      stop("incr should be numeric or the character string \"TACC\"")
+    ##
+    incr <- c("TACC")[iincr]
+  }
+  
+  
+  ##
+  ## Check for levels of confidence interval
+  ##
+  if (!is.numeric(level) | length(level)!=1)
+    stop("parameter 'level' must be a numeric of length 1")
+  if (level <= 0 | level >= 1)
+    stop("parameter 'level': no valid level for confidence interval")
+  ##
+  if (!is.numeric(level.comb) | length(level.comb)!=1)
+    stop("parameter 'level.comb' must be a numeric of length 1")
+  if (level.comb <= 0 | level.comb >= 1)
+    stop("parameter 'level.comb': no valid level for confidence interval")
   
   
   if (sm == "AS") method <- "Inverse"
@@ -87,13 +119,21 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
   method <- c("Inverse", "MH", "Peto")[imeth]
   ##
   if (method == "Peto" & sm != "OR")
-    stop("Peto's method only possible with 'sm=OR'")
+    stop("Peto's method only possible with \"sm=OR\"")
   ##
   if (k.all == 1 & method == "MH"){
     warning("For a single trial, inverse variance method used instead of Mantel Haenszel method.")
     method <- "Inverse"
   }
-  
+
+
+  ##
+  ## Recode integer as numeric:
+  ##
+  if (is.integer(event.e)) event.e <- as.numeric(event.e)
+  if (is.integer(n.e))     n.e     <- as.numeric(n.e)
+  if (is.integer(event.c)) event.c <- as.numeric(event.c)
+  if (is.integer(n.c))     n.c     <- as.numeric(n.c)
   
   ##
   ##
@@ -132,29 +172,92 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
   ##
   sparse <- any(sel)
   ##
+  ## No need to add anything to cell counts for
+  ##  (i)  arcsine difference
+  ##  (ii) Peto method
+  ## as summary measure.
+  ## Accordingly, no warning will be printed.
+  ##
+  warn2 <- !(sm == "AS" | method == "Peto")
+  ##
   if (addincr){
-    i <- rep(incr, k.all)
-    if (warn & sm != "AS")
-      warning(paste(incr, "added to each cell frequency of all studies"))
+    ##
+    if (is.numeric(incr)){
+      incr.e <- rep(incr, k.all)
+      incr.c <- rep(incr, k.all)
+      ##
+      if (warn & warn2 & incr > 0)
+        warning(paste("Increment", incr, "added to each cell frequency of all studies"))
+    }
+    else{
+      if (incr=="TACC"){
+        ##
+        ## Treatment arm continuity correction:
+        ##
+        incr.e <- n.e/(n.e+n.c)
+        incr.c <- n.c/(n.e+n.c)
+        ##
+        if (warn & warn2)
+          warning("Treatment arm continuity correction applied to all studies")
+      }
+    }
+    ##
   }
   else{
     if (sparse){
       if (allincr){
-        i <- rep(incr, k.all)
-        if (warn)
-          warning(paste(incr, "added to each cell frequency of all studies"))
+        ##
+        if (is.numeric(incr)){
+          incr.e <- rep(incr, k.all)
+          incr.c <- rep(incr, k.all)
+          ##
+          if (warn & warn2 & incr > 0)
+            warning(paste("Increment", incr, "added to each cell frequency of all studies"))
+        }
+        else{
+          if (incr=="TACC"){
+            ##
+            ## Treatment arm continuity correction:
+            ##
+            incr.e <- n.e/(n.e+n.c)
+            incr.c <- n.c/(n.e+n.c)
+            ##
+            if (warn & warn2)
+              warning("Treatment arm continuity correction applied to all studies")
+          }
+        }
       }
       else{
         ##
         ## Bradburn, Deeks, Altman, Stata-procedure "metan":
         ## & SAS PROC FREQ (for method="Inverse")
         ##
-        i <- incr*sel
-        if (warn)
-          warning(paste(incr, "added to each cell in 2x2 tables with zero cell frequencies"))
+       ##
+        if (is.numeric(incr)){
+          incr.e <- incr*sel
+          incr.c <- incr*sel
+          ##
+           if (warn & warn2 & incr > 0)
+            warning(paste("Increment", incr, "added to each cell in 2x2 tables with zero cell frequencies"))
+        }
+        else{
+          if (incr=="TACC"){
+            ##
+            ## Treatment arm continuity correction:
+            ##
+            incr.e <- n.e/(n.e+n.c)*sel
+            incr.c <- n.c/(n.e+n.c)*sel
+            ##
+            if (warn & warn2)
+              warning("Treatment arm continuity correction applied to studies with zero cell frequencies")
+          }
+        }
       }
     }
-    else i <- rep(0, k.all)
+    else{
+      incr.e <- rep(0, k.all)
+      incr.c <- rep(0, k.all)
+    }
   }
   
   
@@ -184,8 +287,10 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
       ## 
       ## Cooper & Hedges (1994), p. 251-2
       ## 
-      TE <- log(((n11+i)*(n22+i)) / ((n12+i)*(n21+i)))
-      varTE <- 1/(n11+i) + 1/(n12+i) + 1/(n21+i) + 1/(n22+i)
+      TE <- log(((n11+incr.e)*(n22+incr.c)) /
+                ((n12+incr.e)*(n21+incr.c)))
+      varTE <- (1/(n11+incr.e) + 1/(n12+incr.e) +
+                1/(n21+incr.c) + 1/(n22+incr.c))
     }
     else if (method == "Peto"){
       ## 
@@ -204,12 +309,16 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     ## Cooper & Hedges (1994), p. 247-8
     ## 
     if (!RR.cochrane){
-      TE <- log(((n11+i)/(n1.+i))/((n21+i)/(n2.+i)))
-      varTE <- 1/(n11+i) - 1/(n1.+i) + 1/(n21+i) - 1/(n2.+i)
+      TE <- log(((n11+incr.e)/(n1.+incr.e))/
+                ((n21+incr.c)/(n2.+incr.c)))
+      varTE <- (1/(n11+incr.e) - 1/(n1.+incr.e) +
+                1/(n21+incr.c) - 1/(n2.+incr.c))
       }
     else{
-      TE <- log(((n11+i)/(n1.+2*i))/((n21+i)/(n2.+2*i)))
-      varTE <- 1/(n11+i) - 1/(n1.+2*i) + 1/(n21+i) - 1/(n2.+2*i)
+      TE <- log(((n11+incr.e)/(n1.+2*incr.e))/
+                ((n21+incr.c)/(n2.+2*incr.c)))
+      varTE <- (1/(n11+incr.e) - 1/(n1.+2*incr.e) +
+                1/(n21+incr.c) - 1/(n2.+2*incr.c))
     }
   }
   else if (sm == "RD"){
@@ -217,9 +326,10 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     ## Cooper & Hedges (1994), p. 246-7
     ## 
     ##TE <- (n11+i)/(n1.+2*i) - (n21+i)/(n2.+2*i)
+    ##
     TE <- n11/n1. - n21/n2.
-    varTE <- (n11+i)*(n12+i)/(n1.+2*i)^3 +
-      (n21+i)*(n22+i)/(n2.+2*i)^3
+    varTE <- (n11+incr.e)*(n12+incr.e)/(n1.+2*incr.e)^3 +
+      (n21+incr.c)*(n22+incr.c)/(n2.+2*incr.c)^3
   }
   else if (sm == "AS"){
     ## 
@@ -256,7 +366,8 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     if (!is.logical(MH.exact))
       stop("MH.exact must be of type 'logical'")
     ##
-    i <- i*(!MH.exact)
+    incr.e <- incr.e*(!MH.exact)
+    incr.c <- incr.c*(!MH.exact)
     ##
     if (sm == "OR"){
       ## 
@@ -265,10 +376,10 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
       ## Bradburn, Deeks, Altman, Stata-procedure "metan"
       ## und RevMan 3.1 (MH.exact==FALSE)
       ## 
-      A <- (n11+i)*(n22+i)/(n..+4*i)
-      B <- (n11+i + n22+i)/(n..+4*i)
-      C <- (n12+i)*(n21+i)/(n..+4*i)
-      D <- (n12+i + n21+i)/(n..+4*i)
+      A <- (n11+incr.e)*(n22+incr.c)/(n..+2*incr.e+2*incr.c)
+      B <- (n11+incr.e + n22+incr.c)/(n..+2*incr.e+2*incr.c)
+      C <- (n12+incr.e)*(n21+incr.c)/(n..+2*incr.e+2*incr.c)
+      D <- (n12+incr.e + n21+incr.c)/(n..+2*incr.e+2*incr.c)
       ##
       ## Cooper & Hedges (1994), p. 265-6
       ##
@@ -287,14 +398,16 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
       ## Bradburn, Deeks, Altman, Stata-procedure "metan"
       ## (MH.exact==FALSE)
       ##
-      D <- ((n1.+2*i)*(n2.+2*i)*(n.1+2*i) -
-            (n11+i)*(n21+i)*(n..+4*i))/(n..+4*i)^2
-      R <- (n11+i)*(n2.+2*i)/(n..+4*i)
-      S <- (n21+i)*(n1.+2*i)/(n..+4*i)
+      D <- ((n1.+2*incr.e)*(n2.+2*incr.c)*(n.1+incr.e+incr.c) -
+            (n11+incr.e)*(n21+incr.c)*(n..+2*incr.e+2*incr.c))/
+              (n..+2*incr.e+2*incr.c)^2
+      R <- (n11+incr.e)*(n2.+2*incr.c)/(n..+2*incr.e+2*incr.c)
+      S <- (n21+incr.c)*(n1.+2*incr.e)/(n..+2*incr.e+2*incr.c)
       ##
       w.fixed <- S
       TE.fixed <- log(sum(R, na.rm=TRUE)/sum(S, na.rm=TRUE))
-      varTE.fixed <- sum(D, na.rm=TRUE)/(sum(R, na.rm=TRUE)*sum(S, na.rm=TRUE))
+      varTE.fixed <- sum(D, na.rm=TRUE)/(sum(R, na.rm=TRUE)*
+                              sum(S, na.rm=TRUE))
     }
     else if (sm == "RD"){
       ##
@@ -303,9 +416,10 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
       ## Bradburn, Deeks, Altman, Stata-procedure "metan"
       ## und RevMan 3.1 (MH.exact==FALSE)
       ## 
-      R <- ((n11+i)*(n12+i)*(n2.+2*i)^3 +
-            (n21+i)*(n22+i)*(n1.+2*i)^3)/
-              ((n1.+2*i)*(n2.+2*i)*(n..+4*i)^2)
+      R <- ((n11+incr.e)*(n12+incr.e)*(n2.+2*incr.c)^3 +
+            (n21+incr.c)*(n22+incr.c)*(n1.+2*incr.e)^3)/
+              ((n1.+2*incr.e)*(n2.+2*incr.c)*(n..+2*incr.e+2*incr.c)^2)
+      ##
       S <- n1.*n2./n..
       ##
       w.fixed <- S
@@ -350,9 +464,12 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
               event.c=event.c, n.c=n.c,
               studlab=studlab,
               TE=TE, seTE=sqrt(varTE),
-              w.fixed=w.fixed, w.random=w.random,
-              TE.fixed=TE.fixed, seTE.fixed=sqrt(varTE.fixed),
-              TE.random=TE.random, seTE.random=seTE.random,
+              w.fixed=w.fixed,
+              w.random=w.random,
+              TE.fixed=TE.fixed,
+              seTE.fixed=sqrt(varTE.fixed),
+              TE.random=TE.random,
+              seTE.random=seTE.random,
               k=k, Q=Q, tau=sqrt(tau2),
               Q.CMH=Q.CMH,
               sm=sm, method=method,
@@ -363,8 +480,21 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
               allstudies=allstudies,
               MH.exact=MH.exact,
               RR.cochrane=RR.cochrane,
+              incr.e=incr.e,
+              incr.c=incr.c,
+              level=level,
+              level.comb=level.comb,
+              comb.fixed=comb.fixed,
+              comb.random=comb.random,
+              title=title,
+              complab=complab,
+              outclab=outclab,
               call=match.call(),
               warn=warn)
+  ##
+  if (!missing(byvar)) res$byvar <- byvar
+  if (!missing(bylab)) res$bylab <- bylab
+  res$print.byvar <- print.byvar
   
   class(res) <- c("metabin", "meta")
   
